API_BASE_URL = "https://api.novalad.ai/api/v1"
UPLOAD_ENDPOINT = "/generate/link"
STATUS_ENDPOINT = "/status"
OUTPUT_ENDPOINT = "/output"
PROCESS_ENDPOINT = "/process"
DEFAULT_TIMEOUT = 60
SUPPORTED_FILE_EXTENSIONS = ["pptx","ppt","pdf","doc","docx"]